@@ -5,17 +5,14 @@ from typing import Tuple
 import matplotlib.pyplot as plt
 import numpy as np
 from matplotlib.axes import Axes
-from numpy import ndarray
+from numpy.core._multiarray_umath import ndarray
 from pyFAI.azimuthalIntegrator import AzimuthalIntegrator
-from xpdtools.tools import generate_binner, mask_img
 
-__all__ = ['bg_sub', 'auto_mask', 'integrate', 'vis_chi', 'vis_img']
+__all__ = ['bg_sub', 'integrate', 'vis_chi', 'vis_img']
 
-# default mask_img auto mask setting
-_AUTOMASK_SETTING = dict(
-    alpha=2.0
-)
 # default pyfai integration setting
+from pdfstream.vend.masking import generate_binner, mask_img
+
 _INTEG_SETTING = dict(
     npt=1480,
     correctSolidAngle=False,
@@ -55,36 +52,6 @@ def bg_sub(img: ndarray, bg_img: ndarray, bg_scale: float = None) -> ndarray:
     return img
 
 
-def auto_mask(img: ndarray, ai: AzimuthalIntegrator, mask_setting: dict = None) -> Tuple[ndarray, dict]:
-    """Automatically generate the mask of the image.
-
-    Parameters
-    ----------
-    img : ndarray
-        The 2D diffraction image array.
-
-    ai : AzimuthalIntegrator
-        The AzimuthalIntegrator instance.
-
-    mask_setting : dict
-        The user's modification to auto-masking settings.
-
-    Returns
-    -------
-    mask : ndarray
-        The mask as a boolean array. 0 are good pixels, 1 are masked out.
-
-    _mask_setting : dict
-        The whole mask_setting.
-    """
-    _mask_setting = _AUTOMASK_SETTING.copy()
-    if mask_setting is not None:
-        _mask_setting.update(mask_setting)
-    binner = generate_binner(ai, img.shape)
-    mask = np.invert(mask_img(img, binner, **_mask_setting)).astype(int)
-    return mask, _mask_setting
-
-
 def integrate(
     img: ndarray, ai: AzimuthalIntegrator, mask: ndarray = None, integ_setting: dict = None
 ) -> Tuple[ndarray, dict]:
@@ -122,7 +89,7 @@ def integrate(
     return chi, _integ_setting
 
 
-def vis_img(img: ndarray, mask: ndarray = None, img_setting: dict = None) -> Axes:
+def vis_img(img: ndarray, mask: ndarray = None, img_setting: dict = None, show: bool = True) -> Axes:
     """Visualize the processed image. The color map will be determined by statistics of the pixel values. The color map
     is determined by mean +/- z_score * std.
 
@@ -137,6 +104,9 @@ def vis_img(img: ndarray, mask: ndarray = None, img_setting: dict = None) -> Axe
     img_setting : dict
         The user's modification to imshow kwargs except a special key 'z_score'.
 
+    show : bool
+        If True, show the figure.
+
     Returns
     -------
     ax : Axes
@@ -162,11 +132,12 @@ def vis_img(img: ndarray, mask: ndarray = None, img_setting: dict = None) -> Axe
     ax.set_yticklabels([])
     # color bar with magical settings to make it same size as the plot
     plt.colorbar(img_obj, ax=ax, fraction=0.046, pad=0.04)
-    plt.show(block=False)
+    if show:
+        plt.show(block=False)
     return ax
 
 
-def vis_chi(chi: ndarray, plot_setting: dict = None, unit: str = None) -> Axes:
+def vis_chi(chi: ndarray, plot_setting: dict = None, unit: str = None, show: bool = True) -> Axes:
     """Visualize the chi curve.
 
     Parameters
@@ -180,6 +151,9 @@ def vis_chi(chi: ndarray, plot_setting: dict = None, unit: str = None) -> Axes:
     unit: str
         The unit of the chi data. It affects the label of the plot. If None, no unit.
 
+    show : bool
+        If True, show the figure.
+
     Returns
     -------
     ax : Axes
@@ -193,5 +167,43 @@ def vis_chi(chi: ndarray, plot_setting: dict = None, unit: str = None) -> Axes:
     if unit:
         ax.set_xlabel(_LABEL.get(unit))
     ax.set_ylabel('I (A. U.)')
-    plt.show(block=False)
+    if show:
+        plt.show(block=False)
     return ax
+
+
+def auto_mask(
+    img: ndarray, ai: AzimuthalIntegrator, user_mask: ndarray = None, mask_setting: dict = None
+) -> Tuple[ndarray, dict]:
+    """Automatically generate the mask of the image.
+
+    Parameters
+    ----------
+    img : ndarray
+        The 2D diffraction image array.
+
+    ai : AzimuthalIntegrator
+        The AzimuthalIntegrator instance.
+
+    mask_setting : dict
+        The user's modification to auto-masking settings.
+
+    user_mask : ndarray
+        A mask provided by user. The auto generated mask will be multiplied by this mask.
+
+    Returns
+    -------
+    mask : ndarray
+        The mask as a boolean array. 0 are good pixels, 1 are masked out.
+
+    _mask_setting : dict
+        The whole mask_setting.
+    """
+    if mask_setting is not None:
+        _mask_setting = mask_setting
+    else:
+        _mask_setting = dict()
+    binner = generate_binner(ai, img.shape)
+    tmsk = np.invert(user_mask.astype(bool)) if user_mask is not None else None
+    mask = np.invert(mask_img(img, binner, tmsk=tmsk, **_mask_setting)).astype(int)
+    return mask, _mask_setting
